/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

GtkWidget*
create_saveDialog (void)
{
  GtkWidget *saveDialog;
  GdkPixbuf *saveDialog_icon_pixbuf;
  GtkWidget *dialog_vbox1;
  GtkWidget *hbox30;
  GtkWidget *image25;
  GtkWidget *vbox4;
  GtkWidget *labelSaveDialog;
  GtkWidget *notebookSave;
  GtkWidget *vbox14;
  GtkWidget *notebookResources;
  GtkWidget *vbox15;
  GtkWidget *label75;
  GtkWidget *filechooserwidgetResources;
  GtkWidget *buttonLoadResources;
  GtkWidget *image48;
  GtkWidget *vbox16;
  GtkWidget *label76;
  GtkWidget *hbox65;
  GtkWidget *comboboxentryResources;
  GtkWidget *buttonSaveResources;
  GtkWidget *checkLimitOnVisuData;
  GtkWidget *image49;
  GtkWidget *labelTipsResources;
  GtkWidget *hbox55;
  GtkWidget *imageWarningResources;
  GtkWidget *statusbarResources;
  GtkWidget *labelResources;
  GtkWidget *vbox18;
  GtkWidget *hbox54;
  GtkWidget *image42;
  GtkWidget *label83;
  GtkWidget *hbox66;
  GtkWidget *comboboxentryParameters;
  GtkWidget *buttonSaveParameters;
  GtkWidget *hbox56;
  GtkWidget *imageWarningParameters;
  GtkWidget *statusbarParameters;
  GtkWidget *labelParameters;
  GtkWidget *frame6;
  GtkWidget *vbox17;
  GtkWidget *labelHelp;
  GtkWidget *labelHelpTips;
  GtkWidget *hbox51;
  GtkWidget *image40;
  GtkWidget *label80;
  GtkWidget *closeButtonSave;

  saveDialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (saveDialog), _("Save session"));
  gtk_window_set_position (GTK_WINDOW (saveDialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (saveDialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (saveDialog), 450, -1);
  saveDialog_icon_pixbuf = create_pixbuf ("icone-dialog.png");
  if (saveDialog_icon_pixbuf)
    {
      gtk_window_set_icon (GTK_WINDOW (saveDialog), saveDialog_icon_pixbuf);
      g_object_unref (saveDialog_icon_pixbuf);
    }

  dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(saveDialog));
  gtk_widget_show (dialog_vbox1);

  hbox30 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox30);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox30, TRUE, TRUE, 0);

  image25 = create_pixmap (saveDialog, "save-bandeau.png");
  gtk_widget_show (image25);
  gtk_box_pack_start (GTK_BOX (hbox30), image25, FALSE, FALSE, 0);
  gtk_widget_set_valign(image25, GTK_ALIGN_END);
  gtk_widget_set_margin_start(image25, 10);
  gtk_widget_set_margin_end(image25, 10);
  gtk_widget_set_margin_top(image25, 5);
  gtk_widget_set_margin_bottom(image25, 5);
  
  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox4);
  gtk_box_pack_start (GTK_BOX (hbox30), vbox4, TRUE, TRUE, 0);

  labelSaveDialog = gtk_label_new (_("<span size=\"larger\"><b>Manage configuration files</b></span>"));
  gtk_widget_show (labelSaveDialog);
  gtk_box_pack_start (GTK_BOX (vbox4), labelSaveDialog, FALSE, FALSE, 1);
  gtk_label_set_use_markup (GTK_LABEL (labelSaveDialog), TRUE);

  notebookSave = gtk_notebook_new ();
  gtk_widget_show (notebookSave);
  gtk_box_pack_start (GTK_BOX (vbox4), notebookSave, TRUE, TRUE, 5);
  gtk_widget_set_size_request (notebookSave, 600, -1);

  vbox14 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox14);
  gtk_container_add (GTK_CONTAINER (notebookSave), vbox14);
  gtk_container_set_border_width (GTK_CONTAINER (vbox14), 2);

  notebookResources = gtk_notebook_new ();
  gtk_widget_show (notebookResources);
  gtk_box_pack_start (GTK_BOX (vbox14), notebookResources, TRUE, TRUE, 0);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebookResources), GTK_POS_LEFT);

  vbox15 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_margin_top(vbox15, 15);
  gtk_widget_show (vbox15);
  gtk_container_add (GTK_CONTAINER (notebookResources), vbox15);

  label75 = gtk_label_new (_("<b>Load from file</b>"));
  gtk_widget_show (label75);
  gtk_box_pack_start (GTK_BOX (vbox15), label75, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label75), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label75), 0);
  gtk_widget_set_margin_top(label75, 10);
  gtk_widget_set_margin_start(label75, 25);

  filechooserwidgetResources = gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_OPEN);
  gtk_widget_show (filechooserwidgetResources);
  gtk_box_pack_start (GTK_BOX (vbox15), filechooserwidgetResources, TRUE, TRUE, 0);
  gtk_widget_set_size_request (filechooserwidgetResources, -1, 250);

  buttonLoadResources = gtk_button_new_from_icon_name("document-open",
                                                      GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (buttonLoadResources);
  gtk_box_pack_start (GTK_BOX (vbox15), buttonLoadResources, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (buttonLoadResources, FALSE);
  gtk_widget_set_can_default (buttonLoadResources, TRUE);

  image48 = gtk_image_new_from_icon_name("document-open", GTK_ICON_SIZE_LARGE_TOOLBAR);
  gtk_widget_show (image48);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookResources), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookResources), 0), image48);

  vbox16 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_margin_top(vbox16, 15);
  gtk_widget_show (vbox16);
  gtk_container_add (GTK_CONTAINER (notebookResources), vbox16);

  label76 = gtk_label_new (_("<b>Save to file</b>"));
  gtk_widget_show (label76);
  gtk_box_pack_start (GTK_BOX (vbox16), label76, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label76), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label76), 0.);
  gtk_widget_set_margin_top(label76, 10);
  gtk_widget_set_margin_start(label76, 25);

  hbox65 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox65);
  gtk_box_pack_start (GTK_BOX (vbox16), hbox65, FALSE, FALSE, 0);

  comboboxentryResources = gtk_combo_box_text_new_with_entry();
  gtk_widget_show (comboboxentryResources);
  gtk_box_pack_start (GTK_BOX (hbox65), comboboxentryResources, TRUE, TRUE, 0);

  buttonSaveResources = gtk_button_new_from_icon_name("document-save",
                                                      GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (buttonSaveResources);
  gtk_box_pack_start (GTK_BOX (hbox65), buttonSaveResources, FALSE, FALSE, 5);
  gtk_widget_set_can_default (buttonSaveResources, TRUE);

  checkLimitOnVisuData = gtk_check_button_new_with_mnemonic (_("Export resources related to rendered file only"));
  gtk_widget_show (checkLimitOnVisuData);
  gtk_box_pack_start (GTK_BOX (vbox16), checkLimitOnVisuData, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (checkLimitOnVisuData), 5);
  gtk_widget_set_sensitive (checkLimitOnVisuData, FALSE);

  image49 = gtk_image_new_from_icon_name("document-save", GTK_ICON_SIZE_LARGE_TOOLBAR);
  gtk_widget_show (image49);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookResources), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookResources), 1), image49);

  labelTipsResources = gtk_label_new ("");
  gtk_widget_show (labelTipsResources);
  gtk_box_pack_start (GTK_BOX (vbox14), labelTipsResources, FALSE, FALSE, 0);
  gtk_widget_set_can_focus (labelTipsResources, TRUE);
  gtk_label_set_use_markup (GTK_LABEL (labelTipsResources), TRUE);
  gtk_label_set_justify (GTK_LABEL (labelTipsResources), GTK_JUSTIFY_FILL);
  gtk_label_set_selectable (GTK_LABEL (labelTipsResources), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelTipsResources), 0.);
  gtk_widget_set_margin_top(labelTipsResources, 10);

  hbox55 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox55);
  gtk_box_pack_end (GTK_BOX (vbox14), hbox55, FALSE, FALSE, 0);

  imageWarningResources = gtk_image_new_from_icon_name ("gtk-dialog-warning", GTK_ICON_SIZE_MENU);
  gtk_widget_show (imageWarningResources);
  gtk_box_pack_start (GTK_BOX (hbox55), imageWarningResources, FALSE, FALSE, 0);

  statusbarResources = gtk_statusbar_new ();
  gtk_widget_show (statusbarResources);
  gtk_box_pack_start (GTK_BOX (hbox55), statusbarResources, TRUE, TRUE, 0);
#if GTK_MAJOR_VERSION < 3 && GTK_MINOR_VERSION < 24
  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbarResources), FALSE);
#endif

  labelResources = gtk_label_new (_("<b>Resources</b> <span size=\"smaller\"><i>(values related to rendering aspects)</i></span>"));
  gtk_widget_show (labelResources);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookSave), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookSave), 0), labelResources);
  gtk_label_set_use_markup (GTK_LABEL (labelResources), TRUE);

  vbox18 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox18);
  gtk_container_add (GTK_CONTAINER (notebookSave), vbox18);
  gtk_container_set_border_width (GTK_CONTAINER (vbox18), 2);

  hbox54 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox54);
  gtk_box_pack_start (GTK_BOX (vbox18), hbox54, FALSE, FALSE, 3);

  image42 = gtk_image_new_from_icon_name("document-save", GTK_ICON_SIZE_LARGE_TOOLBAR);
  gtk_widget_show (image42);
  gtk_box_pack_start (GTK_BOX (hbox54), image42, FALSE, FALSE, 0);

  label83 = gtk_label_new (_("<b>Save to file</b>"));
  gtk_widget_show (label83);
  gtk_box_pack_start (GTK_BOX (hbox54), label83, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label83), TRUE);
  gtk_widget_set_margin_top(label83, 10);
  gtk_widget_set_margin_start(label83, 25);
  
  hbox66 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox66);
  gtk_box_pack_start (GTK_BOX (vbox18), hbox66, FALSE, FALSE, 0);

  comboboxentryParameters = gtk_combo_box_text_new_with_entry();
  gtk_widget_set_margin_top(comboboxentryParameters, 15);
  gtk_widget_show (comboboxentryParameters);
  gtk_box_pack_start (GTK_BOX (hbox66), comboboxentryParameters, TRUE, TRUE, 0);

  buttonSaveParameters = gtk_button_new_from_icon_name("document-save", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (buttonSaveParameters);
  gtk_box_pack_start (GTK_BOX (hbox66), buttonSaveParameters, FALSE, FALSE, 5);
  gtk_widget_set_can_default(buttonSaveParameters, TRUE);

  hbox56 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox56);
  gtk_box_pack_end (GTK_BOX (vbox18), hbox56, FALSE, FALSE, 0);

  imageWarningParameters = gtk_image_new_from_icon_name ("gtk-dialog-warning", GTK_ICON_SIZE_MENU);
  gtk_widget_show (imageWarningParameters);
  gtk_box_pack_start (GTK_BOX (hbox56), imageWarningParameters, FALSE, FALSE, 0);

  statusbarParameters = gtk_statusbar_new ();
  gtk_widget_show (statusbarParameters);
  gtk_box_pack_start (GTK_BOX (hbox56), statusbarParameters, TRUE, TRUE, 0);
#if GTK_MAJOR_VERSION < 3 && GTK_MINOR_VERSION < 24
  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbarParameters), FALSE);
#endif

  labelParameters = gtk_label_new (_("<b>Parameters</b> <span size=\"smaller\"><i>(Interface options)</i></span>"));
  gtk_widget_show (labelParameters);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookSave), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookSave), 1), labelParameters);
  gtk_label_set_use_markup (GTK_LABEL (labelParameters), TRUE);

  frame6 = gtk_frame_new (NULL);
  gtk_widget_show (frame6);
  gtk_box_pack_end (GTK_BOX (vbox4), frame6, FALSE, FALSE, 0);

  vbox17 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox17);
  gtk_container_add (GTK_CONTAINER (frame6), vbox17);

  labelHelp = gtk_label_new (_("<span size=\"smaller\">When saving, if you just specify a directory '<span font_desc=\"courier\">dir/</span>', it will be save to '<span font_desc=\"courier\">dir/v_sim.[res][par]</span>' by default, ortherwise speficy a full path.</span>"));
  gtk_widget_show (labelHelp);
  gtk_box_pack_start (GTK_BOX (vbox17), labelHelp, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelHelp), TRUE);
  gtk_label_set_justify (GTK_LABEL (labelHelp), GTK_JUSTIFY_FILL);
  gtk_label_set_line_wrap (GTK_LABEL (labelHelp), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelHelp), 0);
  gtk_widget_set_margin_start(labelHelp, 5); gtk_widget_set_margin_top(labelHelp, 5);

  labelHelpTips = gtk_label_new (_("<span size=\"smaller\"><u>Tips</u>: think to create a <span font_desc=\"courier\">$XDG_CONFIG_HOME/v_sim</span> directory and put your resource and parameter files in it. It is scanned at startup.</span>"));
  gtk_widget_show (labelHelpTips);
  gtk_box_pack_start (GTK_BOX (vbox17), labelHelpTips, FALSE, FALSE, 0);
  gtk_widget_set_can_focus(labelHelpTips, TRUE);
  gtk_label_set_use_markup (GTK_LABEL (labelHelpTips), TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (labelHelpTips), TRUE);
  gtk_label_set_selectable (GTK_LABEL (labelHelpTips), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelHelpTips), 0);
  gtk_widget_set_margin_start(labelHelpTips, 5); gtk_widget_set_margin_top(labelHelpTips, 5);

  hbox51 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox51);
  gtk_frame_set_label_widget (GTK_FRAME (frame6), hbox51);

  image40 = gtk_image_new_from_icon_name("help-browser", GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_widget_show (image40);
  gtk_box_pack_start (GTK_BOX (hbox51), image40, TRUE, TRUE, 0);

  label80 = gtk_label_new (_("Help"));
  gtk_widget_show (label80);
  gtk_box_pack_start (GTK_BOX (hbox51), label80, FALSE, FALSE, 0);

  closeButtonSave = gtk_button_new_from_icon_name("window-close", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (closeButtonSave);
  gtk_dialog_add_action_widget (GTK_DIALOG (saveDialog), closeButtonSave, GTK_RESPONSE_CLOSE);
  gtk_widget_set_can_default(closeButtonSave, TRUE);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (saveDialog, saveDialog, "saveDialog");
  GLADE_HOOKUP_OBJECT_NO_REF (saveDialog, dialog_vbox1, "dialog_vbox1");
  GLADE_HOOKUP_OBJECT (saveDialog, hbox30, "hbox30");
  GLADE_HOOKUP_OBJECT (saveDialog, image25, "image25");
  GLADE_HOOKUP_OBJECT (saveDialog, vbox4, "vbox4");
  GLADE_HOOKUP_OBJECT (saveDialog, labelSaveDialog, "labelSaveDialog");
  GLADE_HOOKUP_OBJECT (saveDialog, notebookSave, "notebookSave");
  GLADE_HOOKUP_OBJECT (saveDialog, vbox14, "vbox14");
  GLADE_HOOKUP_OBJECT (saveDialog, notebookResources, "notebookResources");
  GLADE_HOOKUP_OBJECT (saveDialog, vbox15, "vbox15");
  GLADE_HOOKUP_OBJECT (saveDialog, label75, "label75");
  GLADE_HOOKUP_OBJECT (saveDialog, filechooserwidgetResources, "filechooserwidgetResources");
  GLADE_HOOKUP_OBJECT (saveDialog, buttonLoadResources, "buttonLoadResources");
  GLADE_HOOKUP_OBJECT (saveDialog, image48, "image48");
  GLADE_HOOKUP_OBJECT (saveDialog, vbox16, "vbox16");
  GLADE_HOOKUP_OBJECT (saveDialog, label76, "label76");
  GLADE_HOOKUP_OBJECT (saveDialog, hbox65, "hbox65");
  GLADE_HOOKUP_OBJECT (saveDialog, comboboxentryResources, "comboboxentryResources");
  GLADE_HOOKUP_OBJECT (saveDialog, buttonSaveResources, "buttonSaveResources");
  GLADE_HOOKUP_OBJECT (saveDialog, checkLimitOnVisuData, "checkLimitOnVisuData");
  GLADE_HOOKUP_OBJECT (saveDialog, image49, "image49");
  GLADE_HOOKUP_OBJECT (saveDialog, labelTipsResources, "labelTipsResources");
  GLADE_HOOKUP_OBJECT (saveDialog, hbox55, "hbox55");
  GLADE_HOOKUP_OBJECT (saveDialog, imageWarningResources, "imageWarningResources");
  GLADE_HOOKUP_OBJECT (saveDialog, statusbarResources, "statusbarResources");
  GLADE_HOOKUP_OBJECT (saveDialog, labelResources, "labelResources");
  GLADE_HOOKUP_OBJECT (saveDialog, vbox18, "vbox18");
  GLADE_HOOKUP_OBJECT (saveDialog, hbox54, "hbox54");
  GLADE_HOOKUP_OBJECT (saveDialog, image42, "image42");
  GLADE_HOOKUP_OBJECT (saveDialog, label83, "label83");
  GLADE_HOOKUP_OBJECT (saveDialog, hbox66, "hbox66");
  GLADE_HOOKUP_OBJECT (saveDialog, comboboxentryParameters, "comboboxentryParameters");
  GLADE_HOOKUP_OBJECT (saveDialog, buttonSaveParameters, "buttonSaveParameters");
  GLADE_HOOKUP_OBJECT (saveDialog, hbox56, "hbox56");
  GLADE_HOOKUP_OBJECT (saveDialog, imageWarningParameters, "imageWarningParameters");
  GLADE_HOOKUP_OBJECT (saveDialog, statusbarParameters, "statusbarParameters");
  GLADE_HOOKUP_OBJECT (saveDialog, labelParameters, "labelParameters");
  GLADE_HOOKUP_OBJECT (saveDialog, frame6, "frame6");
  GLADE_HOOKUP_OBJECT (saveDialog, vbox17, "vbox17");
  GLADE_HOOKUP_OBJECT (saveDialog, labelHelp, "labelHelp");
  GLADE_HOOKUP_OBJECT (saveDialog, labelHelpTips, "labelHelpTips");
  GLADE_HOOKUP_OBJECT (saveDialog, hbox51, "hbox51");
  GLADE_HOOKUP_OBJECT (saveDialog, image40, "image40");
  GLADE_HOOKUP_OBJECT (saveDialog, label80, "label80");
  GLADE_HOOKUP_OBJECT (saveDialog, closeButtonSave, "closeButtonSave");

  gtk_widget_grab_default (buttonLoadResources);
  return saveDialog;
}

GtkWidget*
create_observeDialog (void)
{
  GtkWidget *observeDialog;
  GdkPixbuf *observeDialog_icon_pixbuf;
  GtkWidget *dialog_vbox3;
  GtkWidget *hbox20;
  GtkWidget *image15;
  GtkWidget *vbox20;
  GtkWidget *hbox60;
  GtkWidget *radioObserve;
  GSList *radioObserve_group = NULL;
  GtkWidget *hbox21;
  GtkWidget *image16;
  GtkWidget *labelObserve;
  GtkWidget *hboxObserve;
  GtkWidget *label66;
  GtkWidget *radioObserveConstrained;
  GSList *radioObserveConstrained_group = NULL;
  GtkWidget *radioObserveWalker;
  GtkWidget *label67;
  GtkWidget *tableObserve;
  GtkWidget *labelTranslation;
  GtkWidget *labelZoom;
  GtkWidget *spinOmega;
  GtkWidget *spinPhi;
  GtkWidget *spinDx;
  GtkWidget *spinGross;
  GtkWidget *spinTheta;
  GtkWidget *label58;
  GtkWidget *label32;
  GtkWidget *label33;
  GtkWidget *label35;
  GtkWidget *label31;
  GtkWidget *buttonVisuUiOrientationChooser;
  GtkWidget *image51;
  GtkWidget *label34;
  GtkWidget *label36;
  GtkWidget *spinDy;
  GtkWidget *spinPersp;
  GtkWidget *notebookAction;
  GtkWidget *vbox7;
  GtkWidget *hboxPick;
  GtkWidget *radioPick;
  GSList *radioPick_group = NULL;
  GtkWidget *hbox22;
  GtkWidget *image17;
  GtkWidget *labelPick;
  GtkWidget *vpaned1;
  GtkWidget *vbox26;
  GtkWidget *scrolledwindow6;
  GtkWidget *viewportPick;
  GtkWidget *vbox24;
  GtkWidget *pickInfo;
  GtkWidget *pickComment;
  GtkWidget *hbox57;
  GtkWidget *checkDrawDistance;
  GtkWidget *buttonEraseDistances;
  GtkWidget *image54;
  GtkWidget *labelMarks;
  GtkWidget *hboxMarks;
  GtkWidget *buttonSetMarks;
  GtkWidget *image56;
  GtkWidget *buttonEraseMarks;
  GtkWidget *image55;
  GtkWidget *vbox25;
  GtkWidget *label87;
  GtkWidget *vbox27;
  GtkWidget *radioMove;
  GSList *radioMove_group = NULL;
  GtkWidget *hbox59;
  GtkWidget *image44;
  GtkWidget *label111;
  GtkWidget *vbox21;
  GtkWidget *hbox72;
  GtkWidget *label90;
  GtkWidget *radioMovePick;
  GSList *radioMovePick_group = NULL;
  GtkWidget *radioMoveRegion;
  GtkWidget *labelNMoves;
  GtkWidget *label136;
  GtkWidget *tableMovePick;
  GtkWidget *label138;
  GtkWidget *label95;
  GtkWidget *label94;
  GtkWidget *label93;
  GtkWidget *label102;
  GtkWidget *labelOriginalX;
  GtkWidget *labelOriginalY;
  GtkWidget *labelOriginalZ;
  GtkWidget *label125;
  GtkWidget *hboxAddNode;
  GtkWidget *label126;
  GtkWidget *hbox67;
  GtkWidget *label120;
  GtkWidget *label121;
  GtkWidget *labelHorizontalAxe;
  GtkWidget *label123;
  GtkWidget *labelVerticalAxe;
  GtkWidget *label88;
  GtkWidget *frame2;
  GtkWidget *labelInfoObservePick;
  GtkWidget *hbox23;
  GtkWidget *image18;
  GtkWidget *label39;
  GtkWidget *buttonBackToCommandPanel;
  GtkWidget *hbox41;
  GtkWidget *image31;
  GtkWidget *label68;
  GtkAccelGroup *accel_group;
#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 12
  GtkTooltips *tooltips;

  tooltips = gtk_tooltips_new ();
#endif

  accel_group = gtk_accel_group_new ();

  observeDialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (observeDialog), _("Pick and observe session"));
  gtk_window_set_position (GTK_WINDOW (observeDialog), GTK_WIN_POS_CENTER_ON_PARENT);
  observeDialog_icon_pixbuf = create_pixbuf ("icone-dialog.png");
  if (observeDialog_icon_pixbuf)
    {
      gtk_window_set_icon (GTK_WINDOW (observeDialog), observeDialog_icon_pixbuf);
      g_object_unref (observeDialog_icon_pixbuf);
    }

  dialog_vbox3 = gtk_dialog_get_content_area(GTK_DIALOG(observeDialog));
  gtk_widget_show (dialog_vbox3);

  hbox20 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox20);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), hbox20, TRUE, TRUE, 0);

  image15 = create_pixmap (observeDialog, "observe-bandeau.png");
  gtk_widget_show (image15);
  gtk_widget_set_valign(image15, GTK_ALIGN_END);
  gtk_box_pack_start (GTK_BOX (hbox20), image15, FALSE, FALSE, 0);
  /*gtk_widget_set_size_request (image15, -1, 400);*/

  vbox20 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox20);
  gtk_box_pack_start (GTK_BOX (hbox20), vbox20, TRUE, TRUE, 0);

  hbox60 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox60);
  gtk_box_pack_start (GTK_BOX (vbox20), hbox60, FALSE, FALSE, 0);

  radioObserve = gtk_radio_button_new (NULL);
  gtk_widget_show (radioObserve);
  gtk_box_pack_start (GTK_BOX (hbox60), radioObserve, TRUE, TRUE, 0);
  gtk_widget_add_accelerator (radioObserve, "clicked", accel_group,
                              GDK_KEY_o, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioObserve), radioObserve_group);
  radioObserve_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioObserve));

  hbox21 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox21);
  gtk_container_add (GTK_CONTAINER (radioObserve), hbox21);

  image16 = gtk_image_new_from_icon_name("zoom-fit-best", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image16);
  gtk_box_pack_start (GTK_BOX (hbox21), image16, FALSE, FALSE, 0);

  labelObserve = gtk_label_new_with_mnemonic (_("Observe"));
  gtk_widget_show (labelObserve);
  gtk_box_pack_start (GTK_BOX (hbox21), labelObserve, FALSE, FALSE, 0);

  hboxObserve = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxObserve);
  gtk_box_pack_start (GTK_BOX (hbox60), hboxObserve, FALSE, FALSE, 0);

  label66 = gtk_label_new ("(");
  gtk_widget_show (label66);
  gtk_box_pack_start (GTK_BOX (hboxObserve), label66, FALSE, FALSE, 0);

  radioObserveConstrained = gtk_radio_button_new_with_mnemonic (NULL, _("constrained"));
  gtk_widget_show (radioObserveConstrained);
  gtk_box_pack_start (GTK_BOX (hboxObserve), radioObserveConstrained, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, radioObserveConstrained, _("Movement are along meridians when the mouse is dragged along y axis and along parallels when the movement is along x axis."), NULL);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioObserveConstrained), radioObserveConstrained_group);
  radioObserveConstrained_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioObserveConstrained));

  radioObserveWalker = gtk_radio_button_new_with_mnemonic (NULL, _("walker"));
  gtk_widget_show (radioObserveWalker);
  gtk_box_pack_start (GTK_BOX (hboxObserve), radioObserveWalker, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, radioObserveWalker, _("Movements are those of a walking ant on a sphere, when mouse move along y axis, the ant goes strait on, when mouse is dragged along x axis, the ant translates on its right or on its left."), NULL);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioObserveWalker), radioObserveConstrained_group);
  radioObserveConstrained_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioObserveWalker));

  label67 = gtk_label_new (")");
  gtk_widget_show (label67);
  gtk_box_pack_start (GTK_BOX (hboxObserve), label67, FALSE, FALSE, 0);

  tableObserve = gtk_grid_new();
  tool_grid_resize(tableObserve, 3, 7)
  gtk_widget_show (tableObserve);
  gtk_box_pack_start (GTK_BOX (vbox20), tableObserve, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (tableObserve), 3);
  gtk_grid_set_column_spacing(GTK_GRID(tableObserve), 2);

  labelTranslation = gtk_label_new (_("Translations"));
  gtk_widget_show (labelTranslation);
  gtk_grid_attach (GTK_GRID (tableObserve), labelTranslation, 0, 1, 2, 1);
  gtk_label_set_xalign(GTK_LABEL(labelTranslation), 1);
  gtk_widget_set_hexpand(labelTranslation, TRUE);

  labelZoom = gtk_label_new (_("Scale"));
  gtk_widget_show (labelZoom);
  gtk_grid_attach (GTK_GRID (tableObserve), labelZoom, 0, 2, 2, 1);
  gtk_label_set_xalign(GTK_LABEL(labelZoom), 1);
  gtk_widget_set_hexpand(labelZoom, TRUE);

  spinOmega = gtk_spin_button_new_with_range(-360, 360, 5);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinOmega), 0);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinOmega), 1);
  gtk_entry_set_width_chars(GTK_ENTRY(spinOmega), 6);
  gtk_widget_show (spinOmega);
  gtk_grid_attach (GTK_GRID (tableObserve), spinOmega, 5, 0, 1, 1);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinOmega), TRUE);

  spinPhi = gtk_spin_button_new_with_range(-360, 360, 5);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPhi), -50);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinPhi), 1);
  gtk_entry_set_width_chars(GTK_ENTRY(spinPhi), 6);
  gtk_widget_show (spinPhi);
  gtk_grid_attach (GTK_GRID (tableObserve), spinPhi, 3, 0, 1, 1);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinPhi), TRUE);

  spinDx = gtk_spin_button_new_with_range(-3, 3, 0.05);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinDx), 0.5);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinDx), 3);
  gtk_entry_set_width_chars(GTK_ENTRY(spinDx), 6);
  gtk_widget_show (spinDx);
  gtk_grid_attach (GTK_GRID (tableObserve), spinDx, 3, 1, 1, 1);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinDx), TRUE);

  spinGross = gtk_spin_button_new_with_range(0.02, 999, 0.1);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinGross), 1);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinGross), 3);
  gtk_entry_set_width_chars(GTK_ENTRY(spinGross), 6);
  gtk_widget_show (spinGross);
  gtk_grid_attach (GTK_GRID (tableObserve), spinGross, 3, 2, 1, 1);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinGross), TRUE);

  spinTheta = gtk_spin_button_new_with_range(-360, 360, 5);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinTheta), 40);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinTheta), 1);
  gtk_entry_set_width_chars(GTK_ENTRY(spinTheta), 6);
  gtk_widget_show (spinTheta);
  gtk_grid_attach (GTK_GRID (tableObserve), spinTheta, 1, 0, 1, 1);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinTheta), TRUE);

  label58 = gtk_label_new (_("omega:"));
  gtk_widget_show (label58);
  gtk_grid_attach (GTK_GRID (tableObserve), label58, 4, 0, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(label58), 1);
  gtk_widget_set_hexpand(label58, TRUE);

  label32 = gtk_label_new (_("phi:"));
  gtk_widget_show (label32);
  gtk_grid_attach (GTK_GRID (tableObserve), label32, 2, 0, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(label32), 1);
  gtk_widget_set_hexpand(label32, TRUE);

  label33 = gtk_label_new (_("dx:"));
  gtk_widget_show (label33);
  gtk_grid_attach (GTK_GRID (tableObserve), label33, 2, 1, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(label33), 1);
  gtk_widget_set_hexpand(label33, TRUE);

  label35 = gtk_label_new (_("zoom:"));
  gtk_widget_show (label35);
  gtk_grid_attach (GTK_GRID (tableObserve), label35, 2, 2, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(label35), 1);
  gtk_widget_set_hexpand(label35, TRUE);

  label31 = gtk_label_new (_("theta:"));
  gtk_widget_show (label31);
  gtk_grid_attach (GTK_GRID (tableObserve), label31, 0, 0, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(label31), 1);
  gtk_widget_set_hexpand(label31, TRUE);

  buttonVisuUiOrientationChooser = gtk_button_new ();
  gtk_widget_show (buttonVisuUiOrientationChooser);
  gtk_grid_attach (GTK_GRID (tableObserve), buttonVisuUiOrientationChooser, 6, 0, 1, 1);

  image51 = create_pixmap (observeDialog, "axes-button.png");
  gtk_widget_show (image51);
  gtk_container_add (GTK_CONTAINER (buttonVisuUiOrientationChooser), image51);

  label34 = gtk_label_new (_("dy:"));
  gtk_widget_show (label34);
  gtk_grid_attach (GTK_GRID (tableObserve), label34, 4, 1, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(label34), 1);
  gtk_widget_set_hexpand(label34, TRUE);

  label36 = gtk_label_new (_("persp.:"));
  gtk_widget_show (label36);
  gtk_grid_attach (GTK_GRID (tableObserve), label36, 4, 2, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(label36), 1);
  gtk_widget_set_hexpand(label36, TRUE);

  spinDy = gtk_spin_button_new_with_range(-3, 3, 0.05);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinDy), 0.5);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinDy), 3);
  gtk_entry_set_width_chars(GTK_ENTRY(spinDy), 6);
  gtk_widget_show (spinDy);
  gtk_grid_attach (GTK_GRID (tableObserve), spinDy, 5, 1, 1, 1);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinDy), TRUE);

  spinPersp = gtk_spin_button_new_with_range(1.1, 100, 0.5);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPersp), 5);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinPersp), 1);
  gtk_entry_set_width_chars(GTK_ENTRY(spinPersp), 6);
  gtk_widget_show (spinPersp);
  gtk_grid_attach (GTK_GRID (tableObserve), spinPersp, 5, 2, 1, 1);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinPersp), TRUE);

  notebookAction = gtk_notebook_new ();
  gtk_widget_show (notebookAction);
  gtk_box_pack_start (GTK_BOX (vbox20), notebookAction, TRUE, TRUE, 0);

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (notebookAction), vbox7);

  hboxPick = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPick);
  gtk_box_pack_start (GTK_BOX (vbox7), hboxPick, FALSE, FALSE, 0);

  radioPick = gtk_radio_button_new (NULL);
  gtk_widget_show (radioPick);
  gtk_box_pack_start (GTK_BOX (hboxPick), radioPick, TRUE, TRUE, 0);
  gtk_widget_add_accelerator (radioPick, "clicked", accel_group,
                              GDK_KEY_p, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioPick), radioPick_group);
  radioPick_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioPick));

  hbox22 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox22);
  gtk_container_add (GTK_CONTAINER (radioPick), hbox22);

  image17 = gtk_image_new_from_icon_name("applications-graphics", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image17);
  gtk_box_pack_start (GTK_BOX (hbox22), image17, FALSE, FALSE, 0);

  labelPick = gtk_label_new_with_mnemonic (_("Pick"));
  gtk_widget_show (labelPick);
  gtk_box_pack_start (GTK_BOX (hbox22), labelPick, FALSE, FALSE, 0);

  vpaned1 = gtk_vpaned_new ();
  gtk_widget_set_margin_top(vpaned1, 10);
  gtk_widget_set_margin_bottom(vpaned1, 10);
  gtk_widget_show (vpaned1);
  gtk_box_pack_start (GTK_BOX (vbox7), vpaned1, TRUE, TRUE, 0);

  vbox26 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox26);
  gtk_paned_pack1 (GTK_PANED (vpaned1), vbox26, TRUE, FALSE);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow6);
  gtk_box_pack_start (GTK_BOX (vbox26), scrolledwindow6, TRUE, TRUE, 0);
  gtk_widget_set_size_request (scrolledwindow6, -1, 100);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  viewportPick = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewportPick);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), viewportPick);

  vbox24 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox24);
  gtk_container_add (GTK_CONTAINER (viewportPick), vbox24);

  pickInfo = gtk_label_new ("");
  gtk_widget_show (pickInfo);
  gtk_box_pack_start (GTK_BOX (vbox24), pickInfo, FALSE, FALSE, 0);
  gtk_widget_set_can_focus (pickInfo, TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (pickInfo), TRUE);
  gtk_label_set_selectable (GTK_LABEL (pickInfo), TRUE);
  gtk_label_set_xalign(GTK_LABEL(pickInfo), 0);
  gtk_widget_set_margin_start(pickInfo, 15); gtk_widget_set_margin_top(pickInfo, 0);

  pickComment = gtk_label_new ("");
  gtk_widget_show (pickComment);
  gtk_box_pack_start (GTK_BOX (vbox24), pickComment, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (pickComment), GTK_JUSTIFY_CENTER);
  gtk_label_set_line_wrap (GTK_LABEL (pickComment), TRUE);

  hbox57 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox57);
  gtk_box_pack_end (GTK_BOX (vbox26), hbox57, FALSE, FALSE, 0);

  checkDrawDistance = gtk_check_button_new_with_mnemonic (_("Persistent measures"));
  gtk_widget_show (checkDrawDistance);
  gtk_box_pack_start (GTK_BOX (hbox57), checkDrawDistance, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkDrawDistance), TRUE);

  buttonEraseDistances = gtk_button_new ();
  gtk_widget_show (buttonEraseDistances);
  gtk_box_pack_start (GTK_BOX (hbox57), buttonEraseDistances, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonEraseDistances, _("Remove all drawn measurements."), NULL);
  gtk_widget_set_focus_on_click(buttonEraseDistances, FALSE);

  image54 = gtk_image_new_from_icon_name("edit-clear", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image54);
  gtk_container_add (GTK_CONTAINER (buttonEraseDistances), image54);

  labelMarks = gtk_label_new (_("Highlights <span size=\"small\">(none)</span>:"));
  gtk_widget_show (labelMarks);
  gtk_box_pack_start (GTK_BOX (hbox57), labelMarks, TRUE, TRUE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelMarks), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelMarks), 1);
  gtk_widget_set_margin_start(labelMarks, 5); gtk_widget_set_margin_top(labelMarks, 0);

  hboxMarks = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxMarks);
  gtk_box_pack_start (GTK_BOX (hbox57), hboxMarks, FALSE, FALSE, 0);

  buttonSetMarks = gtk_button_new ();
  gtk_widget_show (buttonSetMarks);
  gtk_box_pack_start (GTK_BOX (hboxMarks), buttonSetMarks, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonSetMarks, _("Add highlighted nodes to the list below."), NULL);

  image56 = gtk_image_new_from_icon_name("go-down", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image56);
  gtk_container_add (GTK_CONTAINER (buttonSetMarks), image56);

  buttonEraseMarks = gtk_button_new ();
  gtk_widget_show (buttonEraseMarks);
  gtk_box_pack_end (GTK_BOX (hboxMarks), buttonEraseMarks, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonEraseMarks, _("Remove all highlight marks."), NULL);

  image55 = gtk_image_new_from_icon_name("edit-clear", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image55);
  gtk_container_add (GTK_CONTAINER (buttonEraseMarks), image55);

  vbox25 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox25);
  gtk_paned_pack2 (GTK_PANED (vpaned1), vbox25, TRUE, FALSE);
  gtk_widget_set_size_request (vbox25, -1, 152);

  label87 = gtk_label_new (_("Pick"));
  gtk_widget_show (label87);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookAction), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookAction), 0), label87);

  vbox27 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox27);
  gtk_container_add (GTK_CONTAINER (notebookAction), vbox27);

  radioMove = gtk_radio_button_new (NULL);
  gtk_widget_show (radioMove);
  gtk_box_pack_start (GTK_BOX (vbox27), radioMove, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioMove), radioMove_group);
  radioMove_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioMove));

  hbox59 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox59);
  gtk_container_add (GTK_CONTAINER (radioMove), hbox59);

  image44 = gtk_image_new_from_icon_name("go-jump", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image44);
  gtk_box_pack_start (GTK_BOX (hbox59), image44, FALSE, FALSE, 0);

  label111 = gtk_label_new_with_mnemonic (_("Modify nodes (position, numbers...)"));
  gtk_widget_show (label111);
  gtk_box_pack_start (GTK_BOX (hbox59), label111, FALSE, FALSE, 0);

  vbox21 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_margin_top(vbox21, 10);
  gtk_widget_set_margin_bottom(vbox21, 10);
  gtk_widget_show (vbox21);
  gtk_box_pack_start (GTK_BOX (vbox27), vbox21, TRUE, TRUE, 0);

  hbox72 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox72);
  gtk_box_pack_start (GTK_BOX (vbox21), hbox72, FALSE, FALSE, 0);

  label90 = gtk_label_new (_("<b>Move or delete:</b>"));
  gtk_widget_show (label90);
  gtk_box_pack_start (GTK_BOX (hbox72), label90, TRUE, TRUE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label90), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label90), 0);

  radioMovePick = gtk_radio_button_new_with_mnemonic (NULL, _("picked _node"));
  gtk_widget_show (radioMovePick);
  gtk_box_pack_start (GTK_BOX (hbox72), radioMovePick, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioMovePick), radioMovePick_group);
  radioMovePick_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioMovePick));

  radioMoveRegion = gtk_radio_button_new_with_mnemonic (NULL, _("_selected nodes"));
  gtk_widget_show (radioMoveRegion);
  gtk_box_pack_start (GTK_BOX (hbox72), radioMoveRegion, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioMoveRegion), radioMovePick_group);
  radioMovePick_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioMoveRegion));

  labelNMoves = gtk_label_new ("nb");
  gtk_widget_show (labelNMoves);
  gtk_box_pack_start (GTK_BOX (hbox72), labelNMoves, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelNMoves), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelNMoves), 1);

  label136 = gtk_label_new (_("<span size=\"smaller\">(selected nodes are listed in the pick tab)</span>"));
  gtk_widget_show (label136);
  gtk_box_pack_start (GTK_BOX (vbox21), label136, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label136), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label136), 1);

  tableMovePick = gtk_grid_new();
  tool_grid_resize(tableMovePick, 3, 8);
  gtk_widget_show (tableMovePick);
  gtk_box_pack_start (GTK_BOX (vbox21), tableMovePick, FALSE, FALSE, 0);
  gtk_grid_set_column_spacing(GTK_GRID(tableMovePick), 5);

  label138 = gtk_label_new (_("Specific moving axis:"));
  gtk_widget_show (label138);
  gtk_grid_attach (GTK_GRID (tableMovePick), label138, 0, 1, 1, 1);
  gtk_label_set_use_markup (GTK_LABEL (label138), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label138), 1);
  gtk_widget_set_hexpand(label138, TRUE);
  gtk_widget_set_margin_start(label138, 15); gtk_widget_set_margin_top(label138, 0);

  label95 = gtk_label_new ("<b>z</b>");
  gtk_widget_show (label95);
  gtk_grid_attach (GTK_GRID (tableMovePick), label95, 5, 0, 2, 1);
  gtk_label_set_use_markup (GTK_LABEL (label95), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label95), 0.5);
  gtk_widget_set_margin_start(label95, 0); gtk_widget_set_margin_top(label95, 2);

  label94 = gtk_label_new ("<b>y</b>");
  gtk_widget_show (label94);
  gtk_grid_attach (GTK_GRID (tableMovePick), label94, 3, 0, 2, 1);
  gtk_label_set_use_markup (GTK_LABEL (label94), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label94), 0.5);
  gtk_widget_set_margin_start(label94, 0); gtk_widget_set_margin_top(label94, 2);

  label93 = gtk_label_new ("<b>x</b>");
  gtk_widget_show (label93);
  gtk_grid_attach (GTK_GRID (tableMovePick), label93, 1, 0, 2, 1);
  gtk_label_set_use_markup (GTK_LABEL (label93), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label93), 0.5);
  gtk_widget_set_margin_start(label93, 0); gtk_widget_set_margin_top(label93, 2);

  label102 = gtk_label_new (_("Current pos./trans.:"));
  gtk_widget_show (label102);
  gtk_grid_attach (GTK_GRID (tableMovePick), label102, 0, 2, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(label102), 1);
  gtk_widget_set_hexpand(label102, TRUE);
  gtk_widget_set_margin_start(label102, 15); gtk_widget_set_margin_top(label102, 0);

  labelOriginalX = gtk_label_new ("");
  gtk_widget_show (labelOriginalX);
  gtk_grid_attach (GTK_GRID (tableMovePick), labelOriginalX, 2, 2, 1, 1);
  gtk_widget_set_size_request (labelOriginalX, 35, -1);
  gtk_label_set_use_markup (GTK_LABEL (labelOriginalX), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelOriginalX), 0);

  labelOriginalY = gtk_label_new ("");
  gtk_widget_show (labelOriginalY);
  gtk_grid_attach (GTK_GRID (tableMovePick), labelOriginalY, 4, 2, 1, 1);
  gtk_widget_set_size_request (labelOriginalY, 35, -1);
  gtk_label_set_use_markup (GTK_LABEL (labelOriginalY), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelOriginalY), 0);

  labelOriginalZ = gtk_label_new ("");
  gtk_widget_show (labelOriginalZ);
  gtk_grid_attach (GTK_GRID (tableMovePick), labelOriginalZ, 6, 2, 1, 1);
  gtk_widget_set_size_request (labelOriginalZ, 35, -1);
  gtk_label_set_use_markup (GTK_LABEL (labelOriginalZ), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelOriginalZ), 0);

  label125 = gtk_label_new (_("<b>Add a new node:</b>"));
  gtk_widget_show (label125);
  gtk_box_pack_start (GTK_BOX (vbox21), label125, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label125), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label125), 0);
  gtk_widget_set_margin_start(label125, 0); gtk_widget_set_margin_top(label125, 5);

  hboxAddNode = gtk_hbox_new (FALSE, 5);
  gtk_widget_show (hboxAddNode);
  gtk_box_pack_start (GTK_BOX (vbox21), hboxAddNode, FALSE, FALSE, 0);

  label126 = gtk_label_new (_("position:"));
  gtk_widget_show (label126);
  gtk_box_pack_start (GTK_BOX (hboxAddNode), label126, TRUE, TRUE, 0);
  gtk_label_set_xalign(GTK_LABEL(label126), 1);
  gtk_widget_set_margin_start(label126, 15); gtk_widget_set_margin_top(label126, 0);

  hbox67 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox67);
  gtk_box_pack_end (GTK_BOX (vbox21), hbox67, FALSE, FALSE, 0);

  label120 = gtk_label_new (_("<b>Screen basis set:</b>"));
  gtk_widget_show (label120);
  gtk_box_pack_start (GTK_BOX (hbox67), label120, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label120), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label120), 0);

  label121 = gtk_label_new (_("<i>horiz.</i>"));
  gtk_widget_show (label121);
  gtk_box_pack_start (GTK_BOX (hbox67), label121, TRUE, TRUE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label121), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label121), 1);
  gtk_widget_set_margin_start(label121, 5); gtk_widget_set_margin_top(label121, 0);

  labelHorizontalAxe = gtk_label_new ("");
  gtk_widget_show (labelHorizontalAxe);
  gtk_box_pack_start (GTK_BOX (hbox67), labelHorizontalAxe, FALSE, FALSE, 0);

  label123 = gtk_label_new (_("<i>vert.</i>"));
  gtk_widget_show (label123);
  gtk_box_pack_start (GTK_BOX (hbox67), label123, TRUE, TRUE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label123), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label123), 1);
  gtk_widget_set_margin_start(label123, 5); gtk_widget_set_margin_top(label123, 0);

  labelVerticalAxe = gtk_label_new ("");
  gtk_widget_show (labelVerticalAxe);
  gtk_box_pack_start (GTK_BOX (hbox67), labelVerticalAxe, FALSE, FALSE, 0);

  label88 = gtk_label_new (_("Geometry changes"));
  gtk_widget_show (label88);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookAction), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookAction), 1), label88);

  frame2 = gtk_frame_new (NULL);
  gtk_widget_show (frame2);
  gtk_box_pack_end (GTK_BOX (vbox20), frame2, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 3);

  labelInfoObservePick = gtk_label_new ("");
  gtk_widget_show (labelInfoObservePick);
  gtk_container_add (GTK_CONTAINER (frame2), labelInfoObservePick);
  gtk_label_set_use_markup (GTK_LABEL (labelInfoObservePick), TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (labelInfoObservePick), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelInfoObservePick), 0);
  gtk_widget_set_margin_start(labelInfoObservePick, 2); gtk_widget_set_margin_top(labelInfoObservePick, 0);

  hbox23 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox23);
  gtk_frame_set_label_widget (GTK_FRAME (frame2), hbox23);

  image18 = gtk_image_new_from_icon_name("help-browser", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image18);
  gtk_box_pack_start (GTK_BOX (hbox23), image18, TRUE, TRUE, 0);

  label39 = gtk_label_new (_("Help"));
  gtk_widget_show (label39);
  gtk_box_pack_start (GTK_BOX (hbox23), label39, FALSE, FALSE, 0);

  buttonBackToCommandPanel = gtk_button_new ();
  gtk_widget_show (buttonBackToCommandPanel);
  gtk_dialog_add_action_widget (GTK_DIALOG (observeDialog), buttonBackToCommandPanel, GTK_RESPONSE_CLOSE);
  gtk_widget_set_can_default (buttonBackToCommandPanel, TRUE);

  hbox41 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox41);
  gtk_container_add (GTK_CONTAINER (buttonBackToCommandPanel), hbox41);

  image31 = gtk_image_new_from_icon_name("go-previous", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image31);
  gtk_box_pack_start (GTK_BOX (hbox41), image31, FALSE, FALSE, 0);

  label68 = gtk_label_new_with_mnemonic (_("Back to command panel"));
  gtk_widget_show (label68);
  gtk_box_pack_start (GTK_BOX (hbox41), label68, FALSE, FALSE, 0);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (observeDialog, observeDialog, "observeDialog");
  GLADE_HOOKUP_OBJECT_NO_REF (observeDialog, dialog_vbox3, "dialog_vbox3");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox20, "hbox20");
  GLADE_HOOKUP_OBJECT (observeDialog, image15, "image15");
  GLADE_HOOKUP_OBJECT (observeDialog, vbox20, "vbox20");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox60, "hbox60");
  GLADE_HOOKUP_OBJECT (observeDialog, radioObserve, "radioObserve");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox21, "hbox21");
  GLADE_HOOKUP_OBJECT (observeDialog, image16, "image16");
  GLADE_HOOKUP_OBJECT (observeDialog, labelObserve, "labelObserve");
  GLADE_HOOKUP_OBJECT (observeDialog, hboxObserve, "hboxObserve");
  GLADE_HOOKUP_OBJECT (observeDialog, label66, "label66");
  GLADE_HOOKUP_OBJECT (observeDialog, radioObserveConstrained, "radioObserveConstrained");
  GLADE_HOOKUP_OBJECT (observeDialog, radioObserveWalker, "radioObserveWalker");
  GLADE_HOOKUP_OBJECT (observeDialog, label67, "label67");
  GLADE_HOOKUP_OBJECT (observeDialog, tableObserve, "tableObserve");
  GLADE_HOOKUP_OBJECT (observeDialog, labelTranslation, "labelTranslation");
  GLADE_HOOKUP_OBJECT (observeDialog, labelZoom, "labelZoom");
  GLADE_HOOKUP_OBJECT (observeDialog, spinOmega, "spinOmega");
  GLADE_HOOKUP_OBJECT (observeDialog, spinPhi, "spinPhi");
  GLADE_HOOKUP_OBJECT (observeDialog, spinDx, "spinDx");
  GLADE_HOOKUP_OBJECT (observeDialog, spinGross, "spinGross");
  GLADE_HOOKUP_OBJECT (observeDialog, spinTheta, "spinTheta");
  GLADE_HOOKUP_OBJECT (observeDialog, label58, "label58");
  GLADE_HOOKUP_OBJECT (observeDialog, label32, "label32");
  GLADE_HOOKUP_OBJECT (observeDialog, label33, "label33");
  GLADE_HOOKUP_OBJECT (observeDialog, label35, "label35");
  GLADE_HOOKUP_OBJECT (observeDialog, label31, "label31");
  GLADE_HOOKUP_OBJECT (observeDialog, buttonVisuUiOrientationChooser, "buttonVisuUiOrientationChooser");
  GLADE_HOOKUP_OBJECT (observeDialog, image51, "image51");
  GLADE_HOOKUP_OBJECT (observeDialog, label34, "label34");
  GLADE_HOOKUP_OBJECT (observeDialog, label36, "label36");
  GLADE_HOOKUP_OBJECT (observeDialog, spinDy, "spinDy");
  GLADE_HOOKUP_OBJECT (observeDialog, spinPersp, "spinPersp");
  GLADE_HOOKUP_OBJECT (observeDialog, notebookAction, "notebookAction");
  GLADE_HOOKUP_OBJECT (observeDialog, vbox7, "vbox7");
  GLADE_HOOKUP_OBJECT (observeDialog, hboxPick, "hboxPick");
  GLADE_HOOKUP_OBJECT (observeDialog, radioPick, "radioPick");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox22, "hbox22");
  GLADE_HOOKUP_OBJECT (observeDialog, image17, "image17");
  GLADE_HOOKUP_OBJECT (observeDialog, labelPick, "labelPick");
  GLADE_HOOKUP_OBJECT (observeDialog, vpaned1, "vpaned1");
  GLADE_HOOKUP_OBJECT (observeDialog, vbox26, "vbox26");
  GLADE_HOOKUP_OBJECT (observeDialog, scrolledwindow6, "scrolledwindow6");
  GLADE_HOOKUP_OBJECT (observeDialog, viewportPick, "viewportPick");
  GLADE_HOOKUP_OBJECT (observeDialog, vbox24, "vbox24");
  GLADE_HOOKUP_OBJECT (observeDialog, pickInfo, "pickInfo");
  GLADE_HOOKUP_OBJECT (observeDialog, pickComment, "pickComment");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox57, "hbox57");
  GLADE_HOOKUP_OBJECT (observeDialog, checkDrawDistance, "checkDrawDistance");
  GLADE_HOOKUP_OBJECT (observeDialog, buttonEraseDistances, "buttonEraseDistances");
  GLADE_HOOKUP_OBJECT (observeDialog, image54, "image54");
  GLADE_HOOKUP_OBJECT (observeDialog, labelMarks, "labelMarks");
  GLADE_HOOKUP_OBJECT (observeDialog, hboxMarks, "hboxMarks");
  GLADE_HOOKUP_OBJECT (observeDialog, buttonSetMarks, "buttonSetMarks");
  GLADE_HOOKUP_OBJECT (observeDialog, image56, "image56");
  GLADE_HOOKUP_OBJECT (observeDialog, buttonEraseMarks, "buttonEraseMarks");
  GLADE_HOOKUP_OBJECT (observeDialog, image55, "image55");
  GLADE_HOOKUP_OBJECT (observeDialog, vbox25, "vbox25");
  GLADE_HOOKUP_OBJECT (observeDialog, label87, "label87");
  GLADE_HOOKUP_OBJECT (observeDialog, vbox27, "vbox27");
  GLADE_HOOKUP_OBJECT (observeDialog, radioMove, "radioMove");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox59, "hbox59");
  GLADE_HOOKUP_OBJECT (observeDialog, image44, "image44");
  GLADE_HOOKUP_OBJECT (observeDialog, label111, "label111");
  GLADE_HOOKUP_OBJECT (observeDialog, vbox21, "vbox21");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox72, "hbox72");
  GLADE_HOOKUP_OBJECT (observeDialog, label90, "label90");
  GLADE_HOOKUP_OBJECT (observeDialog, radioMovePick, "radioMovePick");
  GLADE_HOOKUP_OBJECT (observeDialog, radioMoveRegion, "radioMoveRegion");
  GLADE_HOOKUP_OBJECT (observeDialog, labelNMoves, "labelNMoves");
  GLADE_HOOKUP_OBJECT (observeDialog, label136, "label136");
  GLADE_HOOKUP_OBJECT (observeDialog, tableMovePick, "tableMovePick");
  GLADE_HOOKUP_OBJECT (observeDialog, label138, "label138");
  GLADE_HOOKUP_OBJECT (observeDialog, label95, "label95");
  GLADE_HOOKUP_OBJECT (observeDialog, label94, "label94");
  GLADE_HOOKUP_OBJECT (observeDialog, label93, "label93");
  GLADE_HOOKUP_OBJECT (observeDialog, label102, "label102");
  GLADE_HOOKUP_OBJECT (observeDialog, labelOriginalX, "labelOriginalX");
  GLADE_HOOKUP_OBJECT (observeDialog, labelOriginalY, "labelOriginalY");
  GLADE_HOOKUP_OBJECT (observeDialog, labelOriginalZ, "labelOriginalZ");
  GLADE_HOOKUP_OBJECT (observeDialog, label125, "label125");
  GLADE_HOOKUP_OBJECT (observeDialog, hboxAddNode, "hboxAddNode");
  GLADE_HOOKUP_OBJECT (observeDialog, label126, "label126");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox67, "hbox67");
  GLADE_HOOKUP_OBJECT (observeDialog, label120, "label120");
  GLADE_HOOKUP_OBJECT (observeDialog, label121, "label121");
  GLADE_HOOKUP_OBJECT (observeDialog, labelHorizontalAxe, "labelHorizontalAxe");
  GLADE_HOOKUP_OBJECT (observeDialog, label123, "label123");
  GLADE_HOOKUP_OBJECT (observeDialog, labelVerticalAxe, "labelVerticalAxe");
  GLADE_HOOKUP_OBJECT (observeDialog, label88, "label88");
  GLADE_HOOKUP_OBJECT (observeDialog, frame2, "frame2");
  GLADE_HOOKUP_OBJECT (observeDialog, labelInfoObservePick, "labelInfoObservePick");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox23, "hbox23");
  GLADE_HOOKUP_OBJECT (observeDialog, image18, "image18");
  GLADE_HOOKUP_OBJECT (observeDialog, label39, "label39");
  GLADE_HOOKUP_OBJECT (observeDialog, buttonBackToCommandPanel, "buttonBackToCommandPanel");
  GLADE_HOOKUP_OBJECT (observeDialog, hbox41, "hbox41");
  GLADE_HOOKUP_OBJECT (observeDialog, image31, "image31");
  GLADE_HOOKUP_OBJECT (observeDialog, label68, "label68");

  gtk_widget_grab_default (buttonBackToCommandPanel);
  gtk_window_add_accel_group (GTK_WINDOW (observeDialog), accel_group);

  return observeDialog;
}

GtkWidget*
create_infoDialog (void)
{
  GtkWidget *infoDialog;
  GdkPixbuf *infoDialog_icon_pixbuf;
  GtkWidget *dialog_vbox4;
  GtkWidget *vbox10;
  GtkWidget *hbox31;
  GtkWidget *image26;
  GtkWidget *vbox11;
  GtkWidget *table4;
  GtkWidget *label55;
  GtkWidget *label56;
  GtkWidget *label57;
  GtkWidget *labelInfoVersion;
  GtkWidget *labelInfoReleaseDate;
  GtkWidget *labelInfoWebSite;
  GtkWidget *notebookAbout;
  GtkWidget *scrolledwindow4;
  GtkWidget *textviewReadme;
  GtkWidget *label46;
  GtkWidget *scrolledwindow3;
  GtkWidget *textviewAuthors;
  GtkWidget *label47;
  GtkWidget *scrolledwindow2;
  GtkWidget *textviewLicence;
  GtkWidget *labelLicense;
  GtkWidget *vbox23;
  GtkWidget *label116;
  GtkWidget *scrolledwindowPlugins;
  GtkWidget *label115;
  GtkWidget *scrolledwindow7;
  GtkWidget *textviewChangelog;
  GtkWidget *label137;
  GtkWidget *closebutton2;

  infoDialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (infoDialog), _("About V_Sim"));
  gtk_window_set_position (GTK_WINDOW (infoDialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (infoDialog), TRUE);
  infoDialog_icon_pixbuf = create_pixbuf ("icone-about.png");
  if (infoDialog_icon_pixbuf)
    {
      gtk_window_set_icon (GTK_WINDOW (infoDialog), infoDialog_icon_pixbuf);
      g_object_unref (infoDialog_icon_pixbuf);
    }
  gtk_window_set_type_hint (GTK_WINDOW (infoDialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox4 = gtk_dialog_get_content_area(GTK_DIALOG(infoDialog));
  gtk_widget_show (dialog_vbox4);

  vbox10 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox10);
  gtk_box_pack_start (GTK_BOX (dialog_vbox4), vbox10, TRUE, TRUE, 0);

  hbox31 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_margin_bottom(hbox31, 10);
  gtk_widget_show (hbox31);
  gtk_box_pack_start (GTK_BOX (vbox10), hbox31, TRUE, TRUE, 0);

  image26 = create_pixmap (infoDialog, "logo_rectangle.png");
  gtk_widget_show (image26);
  gtk_box_pack_start (GTK_BOX (hbox31), image26, FALSE, FALSE, 0);
  gtk_widget_set_margin_start(image26, 10); gtk_widget_set_margin_top(image26, 10);

  vbox11 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox11);
  gtk_box_pack_start (GTK_BOX (hbox31), vbox11, TRUE, TRUE, 0);

  table4 = gtk_grid_new ();
  tool_grid_resize(table4, 3, 2);
  gtk_widget_show (table4);
  gtk_box_pack_start (GTK_BOX (vbox11), table4, TRUE, FALSE, 0);

  label55 = gtk_label_new (_("<b>Version</b> :"));
  gtk_widget_show (label55);
  gtk_grid_attach (GTK_GRID (table4), label55, 0, 0, 1, 1);
  gtk_label_set_use_markup (GTK_LABEL (label55), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label55), 1);

  label56 = gtk_label_new (_("<b>Release Date</b> :"));
  gtk_widget_show (label56);
  gtk_grid_attach (GTK_GRID (table4), label56, 0, 1, 1, 1);
  gtk_label_set_use_markup (GTK_LABEL (label56), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label56), 1);

  label57 = gtk_label_new (_("<b>Web site</b> :"));
  gtk_widget_show (label57);
  gtk_grid_attach (GTK_GRID (table4), label57, 0, 2, 1, 1);
  gtk_label_set_use_markup (GTK_LABEL (label57), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label57), 1);

  labelInfoVersion = gtk_label_new ("2.99");
  gtk_widget_show (labelInfoVersion);
  gtk_grid_attach (GTK_GRID (table4), labelInfoVersion, 1, 0, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(labelInfoVersion), 0);
  gtk_widget_set_margin_start(labelInfoVersion, 5); gtk_widget_set_margin_top(labelInfoVersion, 0);

  labelInfoReleaseDate = gtk_label_new ("AAAA-MM-JJ");
  gtk_widget_show (labelInfoReleaseDate);
  gtk_grid_attach (GTK_GRID (table4), labelInfoReleaseDate, 1, 1, 1, 1);
  gtk_label_set_xalign(GTK_LABEL(labelInfoReleaseDate), 0);
  gtk_widget_set_margin_start(labelInfoReleaseDate, 5); gtk_widget_set_margin_top(labelInfoReleaseDate, 0);

  labelInfoWebSite = gtk_label_new ("<u>http://</u>");
  gtk_widget_show (labelInfoWebSite);
  gtk_grid_attach (GTK_GRID (table4), labelInfoWebSite, 1, 2, 1, 1);
  gtk_widget_set_can_focus (labelInfoWebSite, TRUE);
  gtk_label_set_use_markup (GTK_LABEL (labelInfoWebSite), TRUE);
  gtk_label_set_selectable (GTK_LABEL (labelInfoWebSite), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelInfoWebSite), 0);
  gtk_widget_set_margin_start(labelInfoWebSite, 5); gtk_widget_set_margin_top(labelInfoWebSite, 0);

  notebookAbout = gtk_notebook_new ();
  gtk_widget_show (notebookAbout);
  gtk_box_pack_start (GTK_BOX (vbox10), notebookAbout, TRUE, TRUE, 0);
  gtk_widget_set_size_request (notebookAbout, 475, 300);

  scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow4);
  gtk_container_add (GTK_CONTAINER (notebookAbout), scrolledwindow4);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_SHADOW_ETCHED_IN);

  textviewReadme = gtk_text_view_new ();
  gtk_widget_show (textviewReadme);
  gtk_container_add (GTK_CONTAINER (scrolledwindow4), textviewReadme);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textviewReadme), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textviewReadme), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textviewReadme), FALSE);
  gtk_text_view_set_left_margin (GTK_TEXT_VIEW (textviewReadme), 2);
  gtk_text_view_set_right_margin (GTK_TEXT_VIEW (textviewReadme), 2);

  label46 = gtk_label_new (_("Readme"));
  gtk_widget_show (label46);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookAbout), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookAbout), 0), label46);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow3);
  gtk_container_add (GTK_CONTAINER (notebookAbout), scrolledwindow3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow3), GTK_SHADOW_ETCHED_IN);

  textviewAuthors = gtk_text_view_new ();
  gtk_widget_show (textviewAuthors);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), textviewAuthors);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textviewAuthors), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textviewAuthors), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textviewAuthors), FALSE);
  gtk_text_view_set_left_margin (GTK_TEXT_VIEW (textviewAuthors), 2);
  gtk_text_view_set_right_margin (GTK_TEXT_VIEW (textviewAuthors), 2);

  label47 = gtk_label_new (_("Authors"));
  gtk_widget_show (label47);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookAbout), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookAbout), 1), label47);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow2);
  gtk_container_add (GTK_CONTAINER (notebookAbout), scrolledwindow2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_SHADOW_ETCHED_IN);

  textviewLicence = gtk_text_view_new ();
  gtk_widget_show (textviewLicence);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), textviewLicence);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textviewLicence), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textviewLicence), FALSE);
  gtk_text_view_set_left_margin (GTK_TEXT_VIEW (textviewLicence), 2);
  gtk_text_view_set_right_margin (GTK_TEXT_VIEW (textviewLicence), 2);

  labelLicense = gtk_label_new (_("License"));
  gtk_widget_show (labelLicense);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookAbout), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookAbout), 2), labelLicense);

  vbox23 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox23);
  gtk_container_add (GTK_CONTAINER (notebookAbout), vbox23);

  label116 = gtk_label_new (_("Loaded plug-ins:"));
  gtk_widget_show (label116);
  gtk_box_pack_start (GTK_BOX (vbox23), label116, FALSE, FALSE, 0);
  gtk_label_set_xalign(GTK_LABEL(label116), 0);
  gtk_widget_set_margin_start(label116, 15); gtk_widget_set_margin_top(label116, 5);

  scrolledwindowPlugins = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindowPlugins);
  gtk_box_pack_start (GTK_BOX (vbox23), scrolledwindowPlugins, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindowPlugins), GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);

  label115 = gtk_label_new (_("Plug-ins"));
  gtk_widget_show (label115);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookAbout), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookAbout), 3), label115);

  scrolledwindow7 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow7);
  gtk_container_add (GTK_CONTAINER (notebookAbout), scrolledwindow7);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow7), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow7), GTK_SHADOW_ETCHED_IN);

  textviewChangelog = gtk_text_view_new ();
  gtk_widget_show (textviewChangelog);
  gtk_container_add (GTK_CONTAINER (scrolledwindow7), textviewChangelog);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textviewChangelog), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textviewChangelog), FALSE);
  gtk_text_view_set_left_margin (GTK_TEXT_VIEW (textviewChangelog), 2);
  gtk_text_view_set_right_margin (GTK_TEXT_VIEW (textviewChangelog), 2);

  label137 = gtk_label_new (_("Changelog"));
  gtk_widget_show (label137);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookAbout), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookAbout), 4), label137);

  closebutton2 = gtk_button_new_from_icon_name("window-close", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (closebutton2);
  gtk_dialog_add_action_widget (GTK_DIALOG (infoDialog), closebutton2, GTK_RESPONSE_CLOSE);
  gtk_widget_set_can_default (closebutton2, TRUE);

  g_signal_connect_swapped ((gpointer) closebutton2, "clicked",
                            G_CALLBACK (gtk_widget_hide),
                            G_OBJECT(infoDialog));

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (infoDialog, infoDialog, "infoDialog");
  GLADE_HOOKUP_OBJECT_NO_REF (infoDialog, dialog_vbox4, "dialog_vbox4");
  GLADE_HOOKUP_OBJECT (infoDialog, vbox10, "vbox10");
  GLADE_HOOKUP_OBJECT (infoDialog, hbox31, "hbox31");
  GLADE_HOOKUP_OBJECT (infoDialog, image26, "image26");
  GLADE_HOOKUP_OBJECT (infoDialog, vbox11, "vbox11");
  GLADE_HOOKUP_OBJECT (infoDialog, table4, "table4");
  GLADE_HOOKUP_OBJECT (infoDialog, label55, "label55");
  GLADE_HOOKUP_OBJECT (infoDialog, label56, "label56");
  GLADE_HOOKUP_OBJECT (infoDialog, label57, "label57");
  GLADE_HOOKUP_OBJECT (infoDialog, labelInfoVersion, "labelInfoVersion");
  GLADE_HOOKUP_OBJECT (infoDialog, labelInfoReleaseDate, "labelInfoReleaseDate");
  GLADE_HOOKUP_OBJECT (infoDialog, labelInfoWebSite, "labelInfoWebSite");
  GLADE_HOOKUP_OBJECT (infoDialog, notebookAbout, "notebookAbout");
  GLADE_HOOKUP_OBJECT (infoDialog, scrolledwindow4, "scrolledwindow4");
  GLADE_HOOKUP_OBJECT (infoDialog, textviewReadme, "textviewReadme");
  GLADE_HOOKUP_OBJECT (infoDialog, label46, "label46");
  GLADE_HOOKUP_OBJECT (infoDialog, scrolledwindow3, "scrolledwindow3");
  GLADE_HOOKUP_OBJECT (infoDialog, textviewAuthors, "textviewAuthors");
  GLADE_HOOKUP_OBJECT (infoDialog, label47, "label47");
  GLADE_HOOKUP_OBJECT (infoDialog, scrolledwindow2, "scrolledwindow2");
  GLADE_HOOKUP_OBJECT (infoDialog, textviewLicence, "textviewLicence");
  GLADE_HOOKUP_OBJECT (infoDialog, labelLicense, "labelLicense");
  GLADE_HOOKUP_OBJECT (infoDialog, vbox23, "vbox23");
  GLADE_HOOKUP_OBJECT (infoDialog, label116, "label116");
  GLADE_HOOKUP_OBJECT (infoDialog, scrolledwindowPlugins, "scrolledwindowPlugins");
  GLADE_HOOKUP_OBJECT (infoDialog, label115, "label115");
  GLADE_HOOKUP_OBJECT (infoDialog, scrolledwindow7, "scrolledwindow7");
  GLADE_HOOKUP_OBJECT (infoDialog, textviewChangelog, "textviewChangelog");
  GLADE_HOOKUP_OBJECT (infoDialog, label137, "label137");
  GLADE_HOOKUP_OBJECT (infoDialog, closebutton2, "closebutton2");

  return infoDialog;
}

GtkWidget*
create_pairsDialog (void)
{
  GtkWidget *pairsDialog;
  GdkPixbuf *pairsDialog_icon_pixbuf;
  GtkWidget *vbox;
  GtkWidget *hbox33;
  GtkWidget *image32;
  GtkWidget *vbox12;
  GtkWidget *labelTitlePairs;
  GtkWidget *hboxPairsModel;
  GtkWidget *notebookPairs;
  GtkWidget *vboxPairsDialog;
  GtkWidget *hbox73;
  GtkWidget *scrolledwindowPairs;
  GtkWidget *viewport1;
  GtkWidget *hboxViewportPairs;
  GtkWidget *label139;
  GtkWidget *vboxDistances;
  GtkWidget *label140;
  GtkWidget *closebutton3;

  pairsDialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (pairsDialog), _("set and customize pairs"));
  pairsDialog_icon_pixbuf = create_pixbuf ("icone-dialog.png");
  if (pairsDialog_icon_pixbuf)
    {
      gtk_window_set_icon (GTK_WINDOW (pairsDialog), pairsDialog_icon_pixbuf);
      g_object_unref (pairsDialog_icon_pixbuf);
    }

  vbox = gtk_dialog_get_content_area(GTK_DIALOG(pairsDialog));
  gtk_widget_show (vbox);

  hbox33 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox33);
  gtk_box_pack_start (GTK_BOX (vbox), hbox33, FALSE, FALSE, 0);

  image32 = create_pixmap (pairsDialog, "liaison-bandeau.png");
  gtk_widget_show (image32);
  gtk_box_pack_start (GTK_BOX (hbox33), image32, FALSE, FALSE, 0);

  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_box_pack_start (GTK_BOX (hbox33), vbox12, TRUE, TRUE, 0);

  labelTitlePairs = gtk_label_new (_("<span size=\"larger\"><b>Set parameters for pairs</b></span>"));
  gtk_widget_show (labelTitlePairs);
  gtk_box_pack_start (GTK_BOX (vbox12), labelTitlePairs, TRUE, TRUE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelTitlePairs), TRUE);
  gtk_label_set_xalign(GTK_LABEL(labelTitlePairs), 0.5);

  hboxPairsModel = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPairsModel);
  gtk_box_pack_end (GTK_BOX (vbox12), hboxPairsModel, FALSE, FALSE, 0);

  notebookPairs = gtk_notebook_new ();
  gtk_widget_show (notebookPairs);
  gtk_box_pack_start (GTK_BOX (vbox), notebookPairs, TRUE, TRUE, 0);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebookPairs), GTK_POS_LEFT);

  vboxPairsDialog = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vboxPairsDialog);
  gtk_container_add (GTK_CONTAINER (notebookPairs), vboxPairsDialog);

  hbox73 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox73);
  gtk_box_pack_start (GTK_BOX (vboxPairsDialog), hbox73, TRUE, TRUE, 0);

  scrolledwindowPairs = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindowPairs);
  gtk_box_pack_start (GTK_BOX (hbox73), scrolledwindowPairs, TRUE, TRUE, 0);
  gtk_widget_set_size_request (scrolledwindowPairs, -1, 150);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindowPairs), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindowPairs), viewport1);

  hboxViewportPairs = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxViewportPairs);
  gtk_container_add (GTK_CONTAINER (viewport1), hboxViewportPairs);

  label139 = gtk_label_new (_("Pairs"));
  gtk_widget_show (label139);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookPairs), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookPairs), 0), label139);
  gtk_label_set_angle (GTK_LABEL (label139), 90);

  vboxDistances = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vboxDistances);
  gtk_container_add (GTK_CONTAINER (notebookPairs), vboxDistances);

  label140 = gtk_label_new (_("Distances"));
  gtk_widget_show (label140);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookPairs), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookPairs), 1), label140);
  gtk_label_set_angle (GTK_LABEL (label140), 90);

  closebutton3 = gtk_button_new_from_icon_name("window-close", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (closebutton3);
  gtk_dialog_add_action_widget (GTK_DIALOG (pairsDialog), closebutton3, GTK_RESPONSE_CLOSE);
  gtk_widget_set_can_default (closebutton3, TRUE);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (pairsDialog, pairsDialog, "pairsDialog");
  GLADE_HOOKUP_OBJECT_NO_REF (pairsDialog, vbox, "vbox");
  GLADE_HOOKUP_OBJECT (pairsDialog, hbox33, "hbox33");
  GLADE_HOOKUP_OBJECT (pairsDialog, image32, "image32");
  GLADE_HOOKUP_OBJECT (pairsDialog, vbox12, "vbox12");
  GLADE_HOOKUP_OBJECT (pairsDialog, labelTitlePairs, "labelTitlePairs");
  GLADE_HOOKUP_OBJECT (pairsDialog, hboxPairsModel, "hboxPairsModel");
  GLADE_HOOKUP_OBJECT (pairsDialog, notebookPairs, "notebookPairs");
  GLADE_HOOKUP_OBJECT (pairsDialog, vboxPairsDialog, "vboxPairsDialog");
  GLADE_HOOKUP_OBJECT (pairsDialog, hbox73, "hbox73");
  GLADE_HOOKUP_OBJECT (pairsDialog, scrolledwindowPairs, "scrolledwindowPairs");
  GLADE_HOOKUP_OBJECT (pairsDialog, viewport1, "viewport1");
  GLADE_HOOKUP_OBJECT (pairsDialog, hboxViewportPairs, "hboxViewportPairs");
  GLADE_HOOKUP_OBJECT (pairsDialog, label139, "label139");
  GLADE_HOOKUP_OBJECT (pairsDialog, vboxDistances, "vboxDistances");
  GLADE_HOOKUP_OBJECT (pairsDialog, label140, "label140");
  GLADE_HOOKUP_OBJECT (pairsDialog, closebutton3, "closebutton3");

  return pairsDialog;
}

GtkWidget*
create_quitDialog (void)
{
  GtkWidget *quitDialog;
  GtkWidget *dialog_vbox5;
  GtkWidget *hbox44;
  GtkWidget *image35;
  GtkWidget *vbox13;
  GtkWidget *label71;
  GtkWidget *checkbuttonHideNextTime;
  GtkWidget *hboxHomedir;
  GtkWidget *image50;
  GtkWidget *label119;
  GtkWidget *buttonAddHomedir;
  GtkWidget *hboxWarning;
  GtkWidget *image46;
  GtkWidget *label114;
  GtkWidget *frame5;
  GtkWidget *label73;
  GtkWidget *hbox45;
  GtkWidget *image36;
  GtkWidget *label72;
  GtkWidget *cancelbutton1;
  GtkWidget *okbutton1;

  quitDialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (quitDialog), _("Quit V_Sim"));
  gtk_window_set_position (GTK_WINDOW (quitDialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (quitDialog), TRUE);
  gtk_window_set_icon_name (GTK_WINDOW (quitDialog), "gtk-dialog-question");
  gtk_window_set_type_hint (GTK_WINDOW (quitDialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox5 = gtk_dialog_get_content_area(GTK_DIALOG(quitDialog));
  gtk_widget_show (dialog_vbox5);

  hbox44 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox44);
  gtk_box_pack_start (GTK_BOX (dialog_vbox5), hbox44, TRUE, TRUE, 0);

  image35 = gtk_image_new_from_icon_name ("gtk-dialog-question", GTK_ICON_SIZE_DIALOG);
  gtk_widget_show (image35);
  gtk_box_pack_start (GTK_BOX (hbox44), image35, FALSE, FALSE, 0);
  gtk_widget_set_margin_start(image35, 6); gtk_widget_set_margin_top(image35, 0);

  vbox13 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox13);
  gtk_box_pack_start (GTK_BOX (hbox44), vbox13, TRUE, TRUE, 0);

  label71 = gtk_label_new (_("<span size=\"larger\"><b>Quit V_Sim?</b></span>"));
  gtk_widget_show (label71);
  gtk_box_pack_start (GTK_BOX (vbox13), label71, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label71), TRUE);
  gtk_widget_set_margin_start(label71, 0); gtk_widget_set_margin_top(label71, 10);

  checkbuttonHideNextTime = gtk_check_button_new_with_mnemonic (_("Don't show this warning again."));
  gtk_widget_show (checkbuttonHideNextTime);
  gtk_box_pack_start (GTK_BOX (vbox13), checkbuttonHideNextTime, TRUE, TRUE, 0);

  hboxHomedir = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox13), hboxHomedir, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hboxHomedir), 15);

  image50 = gtk_image_new_from_icon_name("dialog-warning", GTK_ICON_SIZE_LARGE_TOOLBAR);
  gtk_widget_show (image50);
  gtk_box_pack_start (GTK_BOX (hboxHomedir), image50, FALSE, FALSE, 5);

  label119 = gtk_label_new (_("<i>Can't find a local configuration directory <span font_desc=\"courier\">$XDG_CONFIG_HOME/v_sim</span>. Should one be added?</i>"));
  gtk_widget_show (label119);
  gtk_box_pack_start (GTK_BOX (hboxHomedir), label119, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label119), TRUE);
  gtk_label_set_justify (GTK_LABEL (label119), GTK_JUSTIFY_FILL);
  gtk_label_set_line_wrap (GTK_LABEL (label119), TRUE);

  buttonAddHomedir = gtk_button_new_from_icon_name("list-add", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (buttonAddHomedir);
  gtk_box_pack_start (GTK_BOX (hboxHomedir), buttonAddHomedir, FALSE, FALSE, 0);

  hboxWarning = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox13), hboxWarning, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hboxWarning), 15);

  image46 = gtk_image_new_from_icon_name("dialog-warning", GTK_ICON_SIZE_LARGE_TOOLBAR);
  gtk_widget_show (image46);
  gtk_box_pack_start (GTK_BOX (hboxWarning), image46, FALSE, FALSE, 5);

  label114 = gtk_label_new (_("<i>Can't find a <span font_desc=\"courier\">v_sim.par</span> file with enough write permissions to store the preference, neither in the installation directory nor in the <span font_desc=\"courier\">$XDG_CONFIG_HOME/v_sim</span> one.</i>"));
  gtk_widget_show (label114);
  gtk_box_pack_start (GTK_BOX (hboxWarning), label114, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label114), TRUE);
  gtk_label_set_justify (GTK_LABEL (label114), GTK_JUSTIFY_FILL);
  gtk_label_set_line_wrap (GTK_LABEL (label114), TRUE);

  frame5 = gtk_frame_new (NULL);
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox13), frame5, FALSE, FALSE, 0);

  label73 = gtk_label_new (_("<span size=\"smaller\">If you check the above box but you want to have this warning dialog again before closing, you can change this by editing the parameter file (<span font_desc=\"courier\">v_sim.par</span>). The option is called 'main_confirmQuit'.</span>"));
  gtk_widget_show (label73);
  gtk_container_add (GTK_CONTAINER (frame5), label73);
  gtk_label_set_use_markup (GTK_LABEL (label73), TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (label73), TRUE);
  gtk_label_set_xalign(GTK_LABEL(label73), 0);
  gtk_widget_set_margin_start(label73, 2); gtk_widget_set_margin_top(label73, 2);

  hbox45 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox45);
  gtk_frame_set_label_widget (GTK_FRAME (frame5), hbox45);

  image36 = gtk_image_new_from_icon_name ("gtk-help", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image36);
  gtk_box_pack_start (GTK_BOX (hbox45), image36, TRUE, TRUE, 0);

  label72 = gtk_label_new (_("Help"));
  gtk_widget_show (label72);
  gtk_box_pack_start (GTK_BOX (hbox45), label72, FALSE, FALSE, 0);
  gtk_widget_set_margin_start(label72, 5); gtk_widget_set_margin_top(label72, 0);

  cancelbutton1 = gtk_button_new_with_mnemonic(_("_Cancel"));
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (quitDialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  gtk_widget_set_can_default (cancelbutton1, TRUE);

  okbutton1 = gtk_button_new_from_icon_name("application-exit", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (quitDialog), okbutton1, GTK_RESPONSE_OK);
  gtk_widget_set_can_default (okbutton1, TRUE);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (quitDialog, quitDialog, "quitDialog");
  GLADE_HOOKUP_OBJECT_NO_REF (quitDialog, dialog_vbox5, "dialog_vbox5");
  GLADE_HOOKUP_OBJECT (quitDialog, hbox44, "hbox44");
  GLADE_HOOKUP_OBJECT (quitDialog, image35, "image35");
  GLADE_HOOKUP_OBJECT (quitDialog, vbox13, "vbox13");
  GLADE_HOOKUP_OBJECT (quitDialog, label71, "label71");
  GLADE_HOOKUP_OBJECT (quitDialog, checkbuttonHideNextTime, "checkbuttonHideNextTime");
  GLADE_HOOKUP_OBJECT (quitDialog, hboxHomedir, "hboxHomedir");
  GLADE_HOOKUP_OBJECT (quitDialog, image50, "image50");
  GLADE_HOOKUP_OBJECT (quitDialog, label119, "label119");
  GLADE_HOOKUP_OBJECT (quitDialog, buttonAddHomedir, "buttonAddHomedir");
  GLADE_HOOKUP_OBJECT (quitDialog, hboxWarning, "hboxWarning");
  GLADE_HOOKUP_OBJECT (quitDialog, image46, "image46");
  GLADE_HOOKUP_OBJECT (quitDialog, label114, "label114");
  GLADE_HOOKUP_OBJECT (quitDialog, frame5, "frame5");
  GLADE_HOOKUP_OBJECT (quitDialog, label73, "label73");
  GLADE_HOOKUP_OBJECT (quitDialog, hbox45, "hbox45");
  GLADE_HOOKUP_OBJECT (quitDialog, image36, "image36");
  GLADE_HOOKUP_OBJECT (quitDialog, label72, "label72");
  GLADE_HOOKUP_OBJECT (quitDialog, cancelbutton1, "cancelbutton1");
  GLADE_HOOKUP_OBJECT (quitDialog, okbutton1, "okbutton1");

  return quitDialog;
}

