/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 5
#define gtk_image_new_from_icon_name(name, size) gtk_image_new_from_stock(name,size)
#define gtk_window_set_icon_name(window, name) gtk_window_set_icon(window, (GdkPixbuf*)0)
#endif

#ifndef gtk_widget_ref
#define gtk_widget_ref(Obj) g_object_ref(G_OBJECT(Obj))
#endif
#ifndef gtk_widget_unref
#define gtk_widget_unref g_object_unref
#endif

#if GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION > 10
#define gtk_tooltips_set_tip(T, Obj, tt, ctx) gtk_widget_set_tooltip_text(Obj, tt)
#define gtk_tool_item_set_tooltip(Obj, T, tt, ctx) gtk_tool_item_set_tooltip_text(Obj, tt)
#endif

GtkWidget* create_saveDialog (void);
GtkWidget* create_observeDialog (void);
GtkWidget* create_infoDialog (void);
GtkWidget* create_pairsDialog (void);
GtkWidget* create_quitDialog (void);
