/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  define Q_(String) g_strip_context ((String), gettext (String))
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define Q_(String) g_strip_context ((String), (String))
#  define N_(String) (String)
#endif

#if GTK_MAJOR_VERSION < 3 || (GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 12)
#define gtk_widget_set_margin_start(W, M) gtk_widget_set_margin_left(W, M)
#define gtk_widget_set_margin_end(W, M) gtk_widget_set_margin_right(W, M)
#endif

#if GTK_MAJOR_VERSION < 3 || (GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 4)
#define GTK_GRID(W) GTK_TABLE(W)
#define gtk_grid_new() gtk_table_new(10, 10, FALSE)
#define gtk_grid_attach(G, W, L, T, W, H) gtk_table_attach(G, W, L, L + W, T, T + H, 0, 0, 0, 0)
#endif

#if GTK_MAJOR_VERSION < 3 || (GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 20)
#define gtk_widget_set_focus_on_click(A, B) gtk_button_set_focus_on_click(GTK_BUTTON(A), B)
#endif

#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 15
#define gtk_label_set_xalign(A, B) gtk_misc_set_alignment(GTK_MISC(A), B, 0.5)
#define gtk_label_set_yalign(A, B) gtk_misc_set_alignment(GTK_MISC(A), 0.5, B)
#define gtk_button_new_from_icon_name(A, B) gtk_button_new_with_label(A)
#endif

#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION > 9
#define TOOL_ICON_CANCEL _("_Cancel")
#define TOOL_ICON_SAVE _("_Save")
#define TOOL_ICON_OPEN _("_Open")
#define TOOL_ICON_CLOSE _("_Close")
#else
#define TOOL_ICON_CANCEL GTK_STOCK_CANCEL
#define TOOL_ICON_SAVE GTK_STOCK_SAVE
#define TOOL_ICON_OPEN GTK_STOCK_OPEN
#define TOOL_ICON_CLOSE GTK_STOCK_CLOSE
#endif

#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 4
#define GTK_GRID(A) GTK_TABLE(A)
#define gtk_grid_new() gtk_table_new(1, 1, FALSE)
#define tool_grid_resize(A, B, C) gtk_table_resize(GTK_TABLE(A), B, C)
#define gtk_grid_attach(A, B, X, Y, W, H) gtk_table_attach(A, B, X, X+W, Y, Y+H, 0, 0, 0, 0)
#else
#define tool_grid_resize(A, B, C)
#endif

#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 2

#endif

#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION > 2
#define gtk_hbox_new(A, B) gtk_box_new(GTK_ORIENTATION_HORIZONTAL, B)
#define gtk_vbox_new(A, B) gtk_box_new(GTK_ORIENTATION_VERTICAL, B)
#define gtk_hseparator_new() gtk_separator_new(GTK_ORIENTATION_HORIZONTAL)
#define gtk_vseparator_new() gtk_separator_new(GTK_ORIENTATION_VERTICAL)
#define gtk_hscale_new_with_range(A, B, C) gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, A, B, C)
#define gtk_vscale_new_with_range(A, B, C) gtk_scale_new_with_range(GTK_ORIENTATION_VERTICAL, A, B, C)
#define gtk_hpaned_new() gtk_paned_new(GTK_ORIENTATION_HORIZONTAL)
#define gtk_vpaned_new() gtk_paned_new(GTK_ORIENTATION_VERTICAL)
#endif

#if GTK_MAJOR_VERSION < 3
#define cairo_region_t GdkRegion
#define cairo_region_destroy(Obj) gdk_region_destroy(Obj)
#define gdk_window_get_clip_region(Obj) gdk_drawable_get_clip_region(GDK_DRAWABLE(Obj))
#endif

#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 26 && GTK_MINOR_VERSION > 23
#define gtk_combo_box_text_append(Obj, id, tt) gtk_combo_box_text_append_text(Obj, tt)
#define gtk_combo_box_text_remove(Obj, id) gtk_combo_box_remove_text(GTK_COMBO_BOX(Obj), id)
#endif

#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 24
#define GTK_COMBO_BOX_TEXT(Obj) GTK_COMBO_BOX(Obj)
#define gtk_combo_box_text_new() gtk_combo_box_new_text()
#define gtk_combo_box_text_new_with_entry() gtk_combo_box_entry_new_text()
#define gtk_combo_box_text_append_text(Obj, tt) gtk_combo_box_append_text(Obj, tt)
#define gtk_combo_box_text_append(Obj, id, tt) gtk_combo_box_append_text(Obj, tt)
#define gtk_combo_box_text_remove(Obj, id) gtk_combo_box_remove_text(GTK_COMBO_BOX(Obj), id)
#define gdk_window_get_screen(Obj) gdk_drawable_get_screen(Obj)
#endif

#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 22
#define gtk_dialog_get_content_area(Obj) (Obj->vbox)
#define gtk_dialog_get_action_area(Obj) (Obj->action_area)
#define gtk_color_selection_dialog_get_color_selection(Obj) (Obj->colorsel)
#define GDK_KEY_Home GDK_Home
#define GDK_KEY_0 GDK_0
#define GDK_KEY_1 GDK_1
#define GDK_KEY_2 GDK_2
#define GDK_KEY_3 GDK_3
#define GDK_KEY_4 GDK_4
#define GDK_KEY_5 GDK_5
#define GDK_KEY_6 GDK_6
#define GDK_KEY_7 GDK_7
#define GDK_KEY_8 GDK_8
#define GDK_KEY_9 GDK_9
#define GDK_KEY_f GDK_f
#define GDK_KEY_F GDK_F
#define GDK_KEY_i GDK_i
#define GDK_KEY_I GDK_I
#define GDK_KEY_n GDK_n
#define GDK_KEY_o GDK_o
#define GDK_KEY_p GDK_p
#define GDK_KEY_r GDK_r
#define GDK_KEY_R GDK_R
#define GDK_KEY_s GDK_s
#define GDK_KEY_S GDK_S
#define GDK_KEY_v GDK_v
#define GDK_KEY_V GDK_V
#define GDK_KEY_Menu GDK_Menu
#define GDK_KEY_Escape GDK_Escape
#define GDK_KEY_space GDK_space
#define GDK_KEY_Page_Up GDK_Page_Up
#define GDK_KEY_Page_Down GDK_Page_Down
#define GDK_KEY_Up GDK_Up
#define GDK_KEY_Down GDK_Down
#define GDK_KEY_Right GDK_Right
#define GDK_KEY_Left GDK_Left
#define GDK_KEY_Shift_L GDK_Shift_L
#define GDK_KEY_Shift_R GDK_Shift_R
#define GDK_KEY_Delete GDK_Delete
#define GDK_KEY_BackSpace GDK_BackSpace
#define gtk_widget_is_drawable(Obj) GDK_IS_DRAWABLE(Obj->window)
#define gtk_widget_get_window(Obj) (Obj->window)
#define gtk_widget_get_visible(Obj) GTK_WIDGET_VISIBLE(Obj)
#define gtk_widget_get_allocation(Obj, alloc) {(alloc)->x = Obj->allocation.x; (alloc)->y = Obj->allocation.y; (alloc)->width = Obj->allocation.width; (alloc)->height = Obj->allocation.height;}
#define gtk_widget_set_can_default(Obj, test) {if (test) {GTK_WIDGET_SET_FLAGS(Obj, GTK_CAN_DEFAULT);};}
#define gtk_widget_set_can_focus(Obj, test) {if (test) {GTK_WIDGET_SET_FLAGS(Obj, GTK_CAN_FOCUS);};}
#define gtk_widget_set_window(Obj, win) {Obj->window = win;}
#define gtk_widget_set_realized(Obj, test) {if (test) {GTK_WIDGET_SET_FLAGS(Obj, GTK_REALIZED);};}
#define gtk_widget_set_has_window(Obj, test) {}
#endif
#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 20
#define gtk_widget_style_attach(Obj) {Obj->style = gtk_style_attach(Obj->style, Obj->window);}
#define gtk_widget_get_mapped(Obj) GTK_WIDGET_MAPPED(Obj)
#endif
#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 12
#define gtk_widget_set_tooltip_text(Obj, tt) gtk_tooltips_set_tip(tooltips, Obj, tt, NULL)
#define gtk_tool_item_set_tooltip_text(Obj, tt) gtk_tool_item_set_tooltip(Obj, tooltips, tt, NULL)
#endif
#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 5
#define GTK_STOCK_EDIT GTK_STOCK_PROPERTIES
#endif

/*
 * Public Functions.
 */

/*
 * This function returns a widget in a component created by Glade.
 * Call it with the toplevel widget in the component (i.e. a window/dialog),
 * or alternatively any widget in the component, and the name of the widget
 * you want returned.
 */
GtkWidget*  lookup_widget              (GtkWidget       *widget,
                                        const gchar     *widget_name);


/* Use this function to set the directory containing installed pixmaps. */
void        add_pixmap_directory       (const gchar     *directory);


/*
 * Private Functions.
 */

/* This is used to create the pixmaps used in the interface. */
GtkWidget*  create_pixmap              (GtkWidget       *widget,
                                        const gchar     *filename);

/* This is used to create the pixbufs used in the interface. */
GdkPixbuf*  create_pixbuf              (const gchar     *filename);

/* This is used to set ATK action descriptions. */
void        glade_set_atk_action_description (AtkAction       *action,
                                              const gchar     *action_name,
                                              const gchar     *description);

