#/usr/bin/env python

scene = v_sim.UiMainClass.getDefaultRendering().getGlScene()
scene.dump(v_sim.Dump.png_getStatic(), "out.win.png", 600, 600, None, None)
v_sim.UiMainClass.getCurrentPanel().quit(True)
